import sys
from wizlib.app import WizApp
from wizlib.ui_handler import UIHandler
from wizlib.config_handler import ConfigHandler

from swytchit.command import SwytchitCommand


class SwytchitApp(WizApp):

    base = SwytchitCommand
    name = 'sw'
    handlers = [UIHandler, ConfigHandler]



from wizlib.command import WizCommand
from wizlib.parser import WizParser
from wizlib.ui import Choice, Chooser
from wizlib.command import CommandCancellation


class SwytchitCommand(WizCommand):

    default = 'start'

    # TODO: Move to wizlib
    def confirm(self, verb, *other_actions):  # pragma: nocover
        """Ensure that a command is confirmed by the user"""
        if self.provided('yes'):
            return self.yes
        else:
            def cancel():
                raise CommandCancellation('Cancelled')
            chooser = Chooser(f"{verb}?", 'OK', [
                Choice('OK', '\n', True),
                Choice('cancel', 'c', cancel)
            ])
            for action in other_actions:
                name = action.name if hasattr(action, 'name') else 'other'
                key = action.key if hasattr(action, 'key') else 'o'
                chooser.add_choice(name, key, action)
            choice = self.app.ui.get_option(chooser)
            if type(choice) is bool:
                self.yes = choice
            return choice




from argparse import ArgumentParser
import os
from pathlib import Path
import subprocess
import sys
from tempfile import NamedTemporaryFile
from swytchit.command import SwytchitCommand

import wizlib

from swytchit.error import SwytchitError

RCFILE = '.swytchitrc.sh'


class StartCommand(SwytchitCommand):
    name = 'start'

    @classmethod
    def add_args(self, parser: ArgumentParser):
        parser.add_argument('directory')

    def handle_vals(self):
        super().handle_vals()
        if not sys.stdin.isatty():
            raise SwytchitError('Swytchit only works in interactive tty')
        self.dirpath = Path(self.directory).expanduser().resolve()
        if not (self.dirpath.is_dir()):
            raise SwytchitError(
                'Swytchit requires an existing directory as an argument')
        if not (self.dirpath.is_relative_to(Path.home())):
            raise SwytchitError(
                'Swytchit only operates within user home directory')

    @SwytchitCommand.wrap
    def execute(self):
        shell = self.app.config.get('swytchit-shell') or os.getenv('SHELL')
        parents = [d for d in self.dirpath.parents]
        with NamedTemporaryFile(mode='w+t', delete=False) as rcfile:
            rcfile.write('unset SWYTCHITRC\n')
            for path in reversed([self.dirpath] + parents):
                if path.is_relative_to(Path.home()):
                    if (path / RCFILE).is_file():
                        with open(path / RCFILE) as file:
                            rcfile.write(file.read() + '\n')
            rcfile.seek(0)
            os.environ['SWYTCHITRC'] = rcfile.name
            os.chdir(self.dirpath)
            subprocess.run([shell])
